import datetime
from abc import ABC
from enum import Enum
import datetime
import typesystem as ts

from essencia_core.utils import normalize

definitions = ts.SchemaDefinitions()

class StrEnum(str, Enum):
    def __str__(self):
        return self.value


class Base(ABC):

    def __init__(self, *args, **kwargs):
        super(Base, self).__init__(*args, **kwargs)
        self.request = kwargs.pop('request')
        for k,v in kwargs.items():
            setattr(self, k, v)


    def setup_metadata(self, **kwargs):
        # setting search names and normalizing birthdate
        if hasattr(self, 'fullname'):
            setattr(self, 'search_name', normalize(self.fullname))
            if hasattr(self, 'birthdate'):
                if isinstance(self.birthdate, str):
                    self.birthdate = datetime.date.fromisoformat(self.birthdate)
        elif hasattr(self, 'name'):
            setattr(self, 'search_name', normalize(self.name))

        if not hasattr(self, 'model'):
            setattr(self, 'model', kwargs.get('model') or self.__class__.__name__)

        if not hasattr(self, 'created'):
            setattr(self, 'created', datetime.datetime.now())

        if kwargs.get('request'):
            request = kwargs.get('request')
            setattr(self, 'owner', request.session['auth_user']['email'])
            if hasattr(request.session['auth_user'], 'provider'):
                setattr(self, 'provider_key', request.session['auth_user']['provider']['key'])
            elif hasattr(request.session[ 'auth_user' ], 'assistant'):
                setattr(self, 'assistant_key', request.session[ 'auth_user' ][ 'assistant' ][ 'key' ])
            elif hasattr(request.session[ 'auth_user' ], 'patient'):
                setattr(self, 'patient_key', request.session[ 'auth_user' ][ 'patient' ][ 'key' ])

        if not hasattr(self, 'code'):
            setattr(self, 'code', self.find_code())
        else:
            if getattr(self, 'code') == '':
                setattr(self, 'code', self.find_code())

    def find_code(self):
        if hasattr(self, 'birthdate') and hasattr(self, 'fullname') and hasattr(self, 'gender'):
            birthdate = self.birthdate if isinstance(self.birthdate, datetime.date) else datetime.date.fromisoformat(str(self.birthdate))
            code = f'{birthdate.isoformat().replace("-","")}{self.fullname.split()[0][0]}{self.fullname.split()[-1][0]}{"1" if self.gender.startswith("M") else "2"}'.upper()
        elif hasattr(self, 'patient_key'):
            if hasattr(self, 'provider_key'):
                if not hasattr(self, 'date'):
                    setattr(self, 'date', datetime.date.today())
                    code = f'{str(self.date).replace("-","")}.{self.patient_key}.{self.model}.{self.provider_key}'
        else:
            self.created: datetime.datetime
            code = f'{self.created.date().isoformat().replace("-","")}.{self.created.time().strftime("%H%S")}.{self.__class__.__name__}'
        setattr(self, 'code', code)
        return code

### ABSTRACT MODELS

class BaseModel(ts.Schema, Base, definitions=definitions):

    @property
    def url(self):
        model = getattr(self, 'model', None) or self.__class__.__name__
        return f'/{model.lower()}/k/{self.key}'

    @property
    def json(self):
        return {k: str(v) for (k, v) in self.__dict__.items() if
                v != None and not k in [ 'model', 'table', 'owner', 'created' ]}

    def full_json(self, **kwargs):
        self.setup_metadata(**kwargs)
        x = self.__dict__
        x.update(kwargs)
        return {k: str(v) for (k, v) in x.items() if v != None}


class Profile(BaseModel):
    fullname = ts.String()
    class Gender(StrEnum):
        Male = 'Masculino'
        Female = 'Feminino'
    gender = ts.Choice(choices=Gender)
    birthdate = ts.Date()
    cpf = ts.String(allow_blank=True)
    email = ts.String(allow_blank=True)

    def __eq__(self, other):
        return (self.fullname, self.birthdate, self.gender, self.cpf) == (self.fullname, self.birthdate, self.gender, self.cpf)

    def __hash__(self):
        return hash((self.search_name, self.birthdate, self.gender, self.cpf))

    @property
    def age(self):
        bdate = self.birthdate if isinstance(self.birthdate, datetime.date) else datetime.date.fromisoformat(self.birthdate)
        return ((datetime.date.today() - bdate).days / 365).__round__(1)

    @property
    def age_str(self):
        bdate = self.birthdate if isinstance(self.birthdate, datetime.date) else datetime.date.fromisoformat(self.birthdate)
        age = ((datetime.date.today() - bdate).days / 365).__round__(1)
        if age <= 1:
            return ' '.join([str(age), 'ano'])
        return ' '.join([str(age), 'anos'])

if __name__ == '__main__':
    x = Profile(
        fullname="Daniel Arantes",
        gender='Masculino',
        birthdate=datetime.date.today()
    )
    print(x.json)
    x.setup_metadata()
    print(x.full_json())
    print(x.__hash__())


#!/usr/bin/env python
# coding: utf-8

import contextlib
import typing

from deta import Deta
from typing import Tuple, Dict, Any, Union
from starlette.datastructures import Secret


class DetaCrud:
    def __init__(self, project_key: Union[Secret, str]):
        self.project_key = project_key
        self.deta = Deta(project_key=str(self.project_key))

    @property
    def project_id(self):
        return self.deta.project_id

    def sync_connect(self, table):
        return self.deta.Base(table)


    async def connect(self, table):
        return self.deta.Base(table)

    @contextlib.asynccontextmanager
    async def CountObjects(self, table):
        db = await self.connect(table)
        count = 0
        try:
            count = len(next(db.fetch({})))
        except:
            pass
        finally:
            yield count
            db.client.close()


    @contextlib.asynccontextmanager
    async def Update(self, table: str, data: Dict[str, Any]):
        key = data.get("key")
        if not key:
            raise AttributeError("a key is necessary")
        db = await self.connect(table)
        try:
            yield db.update(data, key)
        except:
            yield
        finally:
            db.client.close()


    @contextlib.asynccontextmanager
    async def ListAll(self, table):
        print(f'looking all for table {table}')
        db = await self.connect(table)
        data = []
        try:
            data = next(db.fetch({}))
        except:
            pass
        finally:
            yield data
            db.client.close()


    @contextlib.asynccontextmanager
    async def CheckCode(self, table, code):
        db = await self.connect(table)
        result = None
        try:
            result = next(db.fetch({'meta.code': code}))[0]
        except:
            pass
        finally:
            yield result
            db.client.close()


    @contextlib.asynccontextmanager
    async def Insert(self, table, data):
        key = data.get("key")
        if not key:
            raise AttributeError("a key is necessary")
        db = await self.connect(table)
        msg = ''
        try:
            db.insert(data)
            msg = 'dados inseridos com sucesso'
        except:
            msg = 'os dados não foram inseridos'
        finally:
            yield msg
            db.client.close()


    @contextlib.asynccontextmanager
    async def Delete(self, table, key):
        db = await self.connect(table)
        try:
            yield db.delete(key=key)
        except:
            yield
        finally:
            db.client.close()


    @contextlib.asynccontextmanager
    async def Put(self, table, data):
        db = await self.connect(table)
        try:
            yield db.put(data)
        except:
            yield
        finally:
            db.client.close()


    @contextlib.asynccontextmanager
    async def Get(self, table, key):
        db = await self.connect(table)
        data = None
        try:
            data = db.get(key=key)
        except:
            pass
        finally:
            yield data
            db.client.close()


    @contextlib.asynccontextmanager
    async def SearchName(self, table, name):
        db = await self.connect(table)
        data = []
        try:
            data = next(db.fetch({'fullname?contains': name}))
        except:
            pass
        finally:
            yield data
            db.client.close()


    @contextlib.asynccontextmanager
    async def Search(self, table: str, query: typing.Dict[str, typing.Any]):
        db = await self.connect(table)
        data = []
        try:
            data = next(db.fetch(query))
        except:
            pass
        finally:
            yield data
            db.client.close()


    @contextlib.asynccontextmanager
    async def First(self, table, query={}):
        db = await self.connect(table)
        try:
            yield next(db.fetch(query))[0]
        except BaseException as e:
            yield
        finally:
            db.client.close()


    @contextlib.asynccontextmanager
    async def Last(self, table: str, query: Dict[str, Any] = dict()):
        db = await self.connect(table)
        try:
            yield next(db.fetch(query))[-1]
        except BaseException as e:
            raise ValueError(e)
        finally:
            db.client.close()


    @contextlib.asynccontextmanager
    async def GetOrCreate(self, table: str, data: Dict[ str, Any ]) -> Tuple[Union[Dict[str, Any]], Union[Dict[str, Any]]]:
        '''
        This function need the code kwarg to perform search in database before saving.
        :param table:
        :param data:
        :return:
        '''
        code = data.get('code', None)
        assert code != None, 'CODE could not be found'
        exist, created = None, None
        base = await self.connect(table)
        try:
            exist = next(base.fetch({'code': code}))[0]
        except:
            created = base.put(data)
        finally:
            yield exist, created
            base.client.close()










import datetime
import typesystem as ts


from essencia_core.models.abstract import StrEnum

### SEARCH MODELS

class VisitInfoAndKey(ts.Schema):
    key = ts.String()
    date = ts.Date(title='Data da Visita')
    class VisitType(StrEnum):
        Initial = 'Inicial'
        NewVisit = 'Nova Consulta'
        Revist = 'Retorno'
        Session = 'Sessão'
    #type = ts.Choice(choices=VisitType, title='Tipo de Visita')
    patient_key = ts.String()
    provider_key = ts.String()
    model = ts.String()


    def __gt__(self, other):
        return (self.date) > (other.date)


class PersonKeyFullname(ts.Schema):
    key = ts.String()
    fullname = ts.String()

class ProfileSearch(ts.Schema):
    name = ts.String(min_length=3, title='Nome', description='pelo menos 3 letras')

class ProfileType(ts.Schema):
    class Options(StrEnum):
        Provider = 'Profissional'
        Assistant = 'Assistente'
        Patient = 'Patient'
    profile_type = ts.Choice(choices=Options)



import typesystem as ts
from essencia_core.models.abstract import Profile, StrEnum

class Assistant(Profile):
    class AssistantCategory(StrEnum):
        Employee = 'Funcionário'
        Studant = 'Estudante'
        Intern = 'Estagiário'
    category = ts.Choice(choices=AssistantCategory, title='Tipo de Assistente')
    salary = ts.Number(minimum=0, title='Salário Mensal')
    identity = ts.String(title='Identidade', description='Registro e Número')



import enum
import typing as tp
import typesystem as ts
from markupsafe import Markup
from essencia_core.models.abstract import StrEnum



class FieldSet:

    class Person(ts.Schema):
        fullname = ts.String(title='Nome Completo')

        class Gender(StrEnum):
            Male = 'Masculino'
            Female = 'Feminino'

        gender = ts.Choice(choices=Gender, title='Gênero')
        birthdate = ts.Date(title='Data de Nascimento')

    class Address(ts.Schema):
        street = ts.String(title='Logradouro')
        house = ts.String(title='Número/Casa/Lote/Quadra')
        complement = ts.String(title='Complemento')
        neiborhood = ts.String(title='Bairro')
        city = ts.String(title='Cidade')

    class Message(ts.Schema):
        message = ts.Text(title='Mensagem')


class Form:
    engine = ts.forms.Jinja2Forms(package='bootstrap4')
    def __init__(self, schema: ts.Schema, values={}, errors: tp.Mapping = {}):
        self.schema = schema
        self.values = values
        self.errors = errors


    def add(self, v):
        assert isinstance(v, ts.Schema), 'o valor de "v" deve uma classe to tipo Typesystem.Schema'


    @property
    def fields(self):
        return Markup(self.engine.Form(self.schema, values=self.values, errors=self.errors).render_fields())



if __name__ == '__main__':
    data = [
        dict(name='Daniel', key='kkhdhf'),
        dict(fullname='Teste', key='kkhdhf'),
        dict(fullname='OUtro', key='kkhdhf'),
        dict(fullname='Lelia', key='kkhdhf'),
        dict(fullname='Ewerton', key='kkhdhf'),
        dict(fullname='Daniel', key='kkhdhf'),

    ]

    print(make_select('patient', 'Paciente', data=data))


import unicodedata
from collections import namedtuple
import typing as tp



def make_people_tuple(data: tp.List[tp.Dict[str, tp.Any]]):
    Person = namedtuple('PersonKeyFullname', 'key fullname')
    cleaned_data = [ {'key': obj["key"], 'fullname': obj["fullname"]} for obj in data if (obj.get('key') and obj.get('fullname'))]
    people = [Person(**obj) for obj in cleaned_data]
    result = sorted(people, key=lambda x: x.fullname)
    return result

def make_people_key_fullname_dict(data: tp.List[tp.Dict[str, tp.Any]]):
    cleaned_data = [ {'key': obj["key"], 'fullname': obj["fullname"]} for obj in data if (obj.get('key') and obj.get('fullname'))]
    result = sorted(cleaned_data, key=lambda x: x['fullname'])
    return result

def make_people_select(id: str, label: str, data: tp.List[tp.Dict[str, tp.Any]]):
    dicts = make_people_key_fullname_dict(data)
    ordered_people = sorted(dicts, key=lambda x: x['fullname'])
    text = f'<select id="{id}">'
    text += f'<label for="{id}">{label}</label>'
    for person in ordered_people:
        text += f'<option value="{person["key"]}">{person["fullname"]}</option>'
    text += '</select>'
    return text


def normalize(input_str):
    def remove_accents(input_str):
        nfkd_form = unicodedata.normalize('NFKD', input_str)
        return u"".join([c for c in nfkd_form if not unicodedata.combining(c)])
    return remove_accents(input_str=input_str).lower().strip()



import datetime as dt
import typesystem as ts
import decimal as dc
from essencia_core.models import abstract


class Service(abstract.BaseModel):
    relation = ts.Object(properties={
        'patient': ts.Object(properties={
            'key': ts.String(),
            'fullname': ts.String(allow_null=True),
            'gender': ts.String(allow_null=True),
            'birthdate': ts.Date(allow_null=True),
        }),
        'provider': ts.Object(properties={
            'key': ts.String(),
        }),
        'date': ts.Date(),
        'financial': ts.Object(properties={
            'value': ts.Decimal(allow_null=True),
            'payment': ts.Object(properties={
                'date': ts.Date(allow_blank=True),
                'value': ts.Decimal(allow_null=True, default=dc.Decimal('0.00')),
                'closed': ts.Boolean(default=False),
                'type': ts.String(),
            }),
        }),
    })
    data = ts.Object(properties={
        'subjective': ts.Text(allow_blank=True),
        'objective': ts.Text(allow_blank=True),
        'assessment': ts.Text(allow_blank=True),
        'plan': ts.Object(properties={
            'instructions': ts.Text(allow_blank=True),
            'prescriptions': ts.Text(allow_blank=True),
            'exam_recipes': ts.Text(allow_blank=True),
            'next_visit': ts.Integer(allow_null=True),
        }),
    })


class VisitFull(abstract.BaseModel):
    patient_key = ts.String()
    provider_key = ts.String()
    main_complaint = ts.Text(title='Queixa Principal', allow_blank=True)
    subjective = ts.Text(title='Descrição dos Sintomas', allow_blank=True)
    objective = ts.Text(title='Descrição dos Sinais', allow_blank=True)
    assessment = ts.Text(title='Avaliação', allow_blank=True)
    plan = ts.Text(title='Plano Terapêutico', allow_blank=True)
    exam_recipe = ts.Text(title='Solicitação de Exames', allow_blank=True)
    prescription = ts.Text(title='Prescrição', allow_blank=True)
    next_visit_date = ts.Date(allow_blank=True)
    next_visit_time = ts.Time(allow_blank=True)


class Pharmacotherapy(abstract.BaseModel):
    medication = ts.String()
    posology = ts.String(allow_blank=True)
    pro = ts.String(allow_blank=True)
    con = ts.String(allow_blank=True)


class Treatment(abstract.BaseModel):
    pharmacotherapy = ts.Array(items=[ts.Reference(to='Pharmacotherapy')], allow_null=True)
    psychotherapy = ts.Text(allow_blank=True)


class Objective(abstract.BaseModel):
    mental = ts.Text(title='Exame Mental', allow_blank=True)
    physical = ts.Text(title='Exame Físico', allow_blank=True)


class PatientProfessionalRelation(abstract.BaseModel):
    patient_key = ts.String(title='Id do Paciente')
    provider_key = ts.String(title='Id do Profissional')
    date = ts.Date(title='Data', allow_null=True)

    def __str__(self):
        if not hasattr(self, 'date'):
            setattr(self, 'date', dt.date.today())
        return '{}_{}_{}'.join([str(self.date).replace('-',''), self.patient_key, self.provider_key])

    class Date(ts.Schema):
        date = ts.Date(title='Data')







class ClinicalNote(abstract.BaseModel):

    class SubjectiveNote(abstract.BaseModel):
        complaints = ts.Text(title='Queixa Principal', allow_blank=True)
        symptoms = ts.Text(title='Sintomas Atuais', allow_blank=True)
        medication = ts.Text(title='Medicações Atuais', allow_blank=True)
        context = ts.Text(title='Contexto Relacionado', allow_blank=True)

    class ObjectiveNote(abstract.BaseModel):
        mental = ts.Text(title='Exame Mental', allow_blank=True)
        physical = ts.Text(title='Exame Físico', allow_blank=True)

    class Vitals(abstract.BaseModel):
        weight = ts.Float(minimum=0, maximum=250, description='Kg', allow_null=True)
        heigth = ts.Float(minimum=0, maximum=210, allow_null=True)
        hr = ts.Integer(minimum=0, maximum=300, description='batimentos por minuto', allow_null=True)
        bp = ts.String(title='pressão sanguínea', description='mmHg', allow_blank=True)




class Visit(abstract.BaseModel):
    patient_key = ts.String(title='Id do Paciente')
    provider_key = ts.String(title='Id do Profissional')


    class SubjectiveNotes(abstract.BaseModel):
        complaints = ts.Text(title='Queixa Principal', allow_blank=True)
        symptoms = ts.Text(title='Sintomas Atuais', allow_blank=True)
        medication = ts.Text(title='Medicações Atuais', allow_blank=True)
        context = ts.Text(title='Contexto Relacionado', allow_blank=True)



class Episode(abstract.BaseModel):
    patient_key = ts.String(title='Paciente', description='patient_key')
    owner = ts.String(allow_blank=True, title='Proprietário')
    subjective = ts.Text(allow_blank=True, title='Dados subjetivos')
    context = ts.Text(allow_blank=True, title='Contexto do episódio')
    objective = ts.Text(allow_blank=True, title='Dados objetivos')
    complement = ts.Text(allow_blank=True, title='Informações complementares')
    date = ts.Date(allow_null=True, title='Data do episódio')
    age = ts.Float(allow_null=True, title='Idade no episódio')



    class Input(abstract.BaseModel):
        subjective = ts.Text(allow_blank=True, title='Dados subjetivos')
        context = ts.Text(allow_blank=True, title='Contexto do episódio')
        objective = ts.Text(allow_blank=True, title='Dados objetivos')
        complement = ts.Text(allow_blank=True, title='Informações complementares')
        date = ts.Date(allow_null=True, title='Data do episódio')
        age = ts.Float(allow_null=True, title='Idade no episódio')

    class Visit(abstract.BaseModel):
        subjective = ts.Text(allow_blank=True, title='Dados subjetivos')
        context = ts.Text(allow_blank=True, title='Contexto do episódio')
        objective = ts.Text(allow_blank=True, title='Dados objetivos')
        complement = ts.Text(allow_blank=True, title='Informações complementares')
        assessment = ts.Text(allow_blank=True, title='Avaliação')
        plan = ts.Text(allow_blank=True, title='Plano Terapêutico')
        prescription = ts.Text(allow_blank=True, title='Prescrição')
        exam_recipe = ts.Text(allow_blank=True, title='Solicitação de exames')
        next_visit = ts.Date(allow_blank=True, title='Próxima Consulta')



class ClinicalRecord(abstract.BaseModel):
    patient_key = ts.String()
    provider_key = ts.String()
    subjective = ts.Text(allow_blank=True, title='Dados subjetivos')
    context = ts.Text(allow_blank=True, title='Contexto do episódio')
    weight = ts.Float(minimum=0, maximum=250, description='Kg', allow_null=True)
    hr = ts.Integer(minimum=0, maximum=300, description='batimentos por minuto', allow_null=True)
    bp = ts.String(title='pressão sanguínea', description='mmHg', allow_blank=True)
    mental = ts.Text(title='Exame Mental', allow_blank=True)
    physical = ts.Text(title='Exame Físico', allow_blank=True)
    complement = ts.Text(allow_blank=True, title='Informações complementares')
    assessment = ts.Text(allow_blank=True, title='Avaliação')
    plan = ts.Text(allow_blank=True, title='Plano Terapêutico')
    prescription = ts.Text(allow_blank=True, title='Prescrição')
    exam_recipe = ts.Text(allow_blank=True, title='Solicitação de exames')
    next_visit_days = ts.Integer(allow_null=True, title='Dias para próxima visita')

    class Visit(abstract.BaseModel):
        subjective = ts.Text(allow_blank=True, title='Dados subjetivos')
        context = ts.Text(allow_blank=True, title='Contexto do episódio')
        weight = ts.Float(minimum=0, maximum=250, description='Kg', allow_null=True)
        hr = ts.Integer(minimum=0, maximum=300, description='batimentos por minuto', allow_null=True)
        bp = ts.String(title='pressão sanguínea', description='mmHg', allow_blank=True)
        mental = ts.Text(title='Exame Mental', allow_blank=True)
        physical = ts.Text(title='Exame Físico', allow_blank=True)
        complement = ts.Text(allow_blank=True, title='Informações complementares')
        assessment = ts.Text(allow_blank=True, title='Avaliação')
        plan = ts.Text(allow_blank=True, title='Plano Terapêutico')
        prescription = ts.Text(allow_blank=True, title='Prescrição')
        exam_recipe = ts.Text(allow_blank=True, title='Solicitação de exames')
        next_visit_days = ts.Integer(allow_null=True, title='Dias para próxima visita')


class VisitBase(ts.Schema):
    relation = ts.Object(properties={
        'patient': ts.Object(properties={
            'key': ts.String(),
            'fullname': ts.String(),
        }),
        'provider': ts.Object(properties={
            'key': ts.String(),
            'fullname': ts.String(),
        })
    })
    object = ts.Object(properties={
        'date': ts.Date(default=dt.date.today()),
        'model': ts.String(allow_blank=True),
        'code': ts.String(allow_blank=True)
    })


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['essencia_core',
 'essencia_core.db',
 'essencia_core.db.deta',
 'essencia_core.forms',
 'essencia_core.models',
 'essencia_core.utils']

package_data = \
{'': ['*']}

install_requires = \
['Jinja2>=3.0.1,<4.0.0',
 'MarkupSafe>=2.0.1,<3.0.0',
 'bootstrap4>=0.1.0,<0.2.0',
 'deta>=0.8,<0.9',
 'starlette>=0.14.2,<0.15.0',
 'typesystem>=0.2.4,<0.3.0',
 'uvicorn>=0.14.0,<0.15.0']

setup_kwargs = {
    'name': 'essencia-core',
    'version': '0.1.4',
    'description': '',
    'long_description': None,
    'author': 'arantesdv',
    'author_email': 'arantesdv@me.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import typesystem as ts

from essencia_core.models.abstract import StrEnum, Profile

class Provider(Profile):
    class Profession(StrEnum):
        Medicine = 'Medicina'
        Psychology = 'Psicologia'
        Physiotherapy = 'Fisioterapia'
        Nurse = 'Enfermagem'
    profession = ts.Choice(choices=Profession)
    licence = ts.String(title='Registro Profissional')
    graduation = ts.Integer(title='Ano de Graduação')
